<?php
//  ---------------------------------------------------------------------------------
//    _____   _             _                   _   _____
//   |  __ \ | |           | |    /\           (_) |  __ \
//   | |__) || |__    __ _ | |   /  \    _ __   _  | |__) |_ __  ___
//   |  ___/ | '_ \  / _` || |  / /\ \  | '_ \ | | |  ___/| '__|/ _ \
//   | |     | | | || (_| || | / ____ \ | |_) || | | |    | |  | (_) |
//   |_|     |_| |_| \__,_||_|/_/    \_\| .__/ |_| |_|    |_|   \___/
//                                      | |
//                                      |_|
//   PhalApi Pro 专业版
//   广州果创网络科技有限公司
//
//  ---------------------------------------------------------------------------------
//
//   一、协议的许可和权利
//    1. 您可以在完全遵守本协议的基础上，将本软件应用于商业用途；
//    2. 您可以在协议规定的约束和限制范围内修改本产品源代码或界面风格以适应您的要求；
//    3. 您拥有使用本产品中的全部内容资料、商品信息及其他信息的所有权，并独立承担与其内容相关的
//       法律义务；
//    4. 获得商业授权之后，您可以将本软件应用于商业用途，自授权时刻起，在技术支持期限内拥有通过
//       指定的方式获得指定范围内的技术支持服务；
//
//   二、协议的约束和限制
//    1. 未获商业授权之前，禁止将本软件用于商业用途（包括但不限于企业法人经营的产品、经营性产品
//       以及以盈利为目的或实现盈利产品）；
//    2. 未获商业授权之前，禁止在本产品的整体或在任何部分基础上发展任何派生版本、修改版本或第三
//       方版本用于重新开发；
//    3. 如果您未能遵守本协议的条款，您的授权将被终止，所被许可的权利将被收回并承担相应法律责任；
//
//   三、有限担保和免责声明
//    1. 本软件及所附带的文件是作为不提供任何明确的或隐含的赔偿或担保的形式提供的；
//    2. 用户出于自愿而使用本软件，您必须了解使用本软件的风险，在尚未获得商业授权之前，我们不承
//       诺提供任何形式的技术支持、使用担保，也不承担任何因使用本软件而产生问题的相关责任；
//    3. 广州果创网络科技有限公司不对使用本产品构建的商城中的内容信息承担责任，但在不侵犯用户隐
//       私信息的前提下，保留以任何方式获取用户信息及商品信息的权利；
//
//   有关本产品最终用户授权协议、商业授权与技术服务的详细内容，均由广州果创网络科技有限公司独家
//   提供。广州果创网络科技有限公司拥有在不事先通知的情况下，修改授权协议的权力，修改后的协议对
//   改变之日起的新授权用户生效。电子文本形式的授权协议如同双方书面签署的协议一样，具有完全的和
//   等同的法律效力。您一旦开始修改、安装或使用本产品，即被视为完全理解并接受本协议的各项条款，
//   在享有上述条款授予的权力的同时，受到相关的约束和限制。协议许可范围以外的行为，将直接违反本
//   授权协议并构成侵权，我们有权随时终止授权，责令停止损害，并保留追究相关责任的权力。
//
//  ---------------------------------------------------------------------------------

namespace Base\Domain;

use Base\Common\ApiInfoUtil;
use Base\Model\Rights as RightsModel;
use Mall\Domain\Product;

class Rights {

    /**
     * 检测应用对接口的权限
     * 优先级：开发者应用 -> 开发者账号 -> 账号类型。最短路径判断
     */
    public function checkAppRights($appKey, $service) {
        $serviceLower = strtolower($service);
        // 第一关，寻找app_key的权限配置
        
        $where = 'service_lower = ? AND app_key = ?';
        $whereParams = array($serviceLower, $appKey);
        $rights = RightsModel::model()->getData($where, $whereParams, 'is_allow');
        if ($rights) {
            return $rights['is_allow'] == 1 ? true : false;
        }
        
        // 第二关，寻找开发者账号的权限配置
        $appsDomain = new Apps();
        $app = $appsDomain->getApp($appKey, 'uid');
        if ($app && $app['uid'] > 0) {
            $where = 'service_lower = ? AND uid = ?';
            $whereParams = array($serviceLower, $app['uid']);
            $rights = RightsModel::model()->getData($where, $whereParams, 'is_allow');
            if ($rights) {
                return $rights['is_allow'] == 1 ? true : false;
            }
            
            // 第三关，寻找开发者类型的权限配置
            $useDomain = new User();
            $level = $useDomain->getMemberLevel($app['uid']);
            // 如果是管理员，默认有接口权限
            if ($level >= UserType::MEMBER_LEVEL_ADMIN) {
                return true;
            }
            // 对于开发者（或者会员），进行权限判断
            if ($level > UserType::MEMBER_LEVEL_USER) {
                $where = 'service_lower = ? AND member_type = ?';
                $whereParams = array($serviceLower, $level);
                $rights = RightsModel::model()->getData($where, $whereParams, 'is_allow');
                if ($rights) {
                    return $rights['is_allow'] == 1 ? true : false;
                }
            }
        }
        
        // 通关！寻找最后宝藏。没有配置，默认接口权限
        return \PhalApi\DI()->config->get('app.project.default_app_api_rigths_is_allow') ? true : false;
    }

    /**
     * 分配权限
     */
    public function assignRights($appKey, $service) {
        $model = new RightsModel();
        return $model->assginRights($appKey, $service);
    }

    /**
     * 移除权限
     */
    public function removeRights($appKey, $service, $adminUid = null) {
        if ($adminUid === null) {
            $adminUid = \PhalApi\DI()->context->getUid();
        }
        if(!ApiInfoUtil::checkApiIsExist($service)){
            return false;
        }
        $model = new RightsModel();
        $data = $model->isExistRight($appKey, $service);
        if(empty($data)){
            return $model->removeRights($appKey, $service, $adminUid);
        }else{
            return $model->update($data['id'], array(
                'app_key' => $appKey,
                'service' => $service,
                'is_allow' => 0,
                'admin_uid' => $adminUid,
                'add_time' => date('Y-m-d H:i:s'),
            ));
        }

    }

    /**
     * 获取应用的全部权限
     */
    public function getRightsForApp($appKey) {
        $model = new RightsModel();
        return $model->getRightsForApp($appKey);

        $rs = array();
        foreach ($rows as $it) {
            $rs[$it['service']] = $it['is_allow'];
        }

        return $rs;
    }

    public function listAllRightsForDeveloperType() {
        // 全部开发者类型
        $devTypes = UserType::getDeveloperLevelDropList(false);

        // 全部接口
        $rsDomain = new RightsService();
        $apis = $rsDomain->listAllRightsService();

        // 规则配置
        $rights = RightsModel::model()->getList('member_type <> 0', array(), 'member_type,service_lower,is_allow', NULL, 1, 10000);
        $rightsMap = array();
        foreach ($rights as $it) {
            if (!isset($rightsMap[$it['service_lower']])) {
                $rightsMap[$it['service_lower']] = array();
            }
            $rightsMap[$it['service_lower']][$it['member_type']] = $it['is_allow'] == 1 ? true : false;
        }

        $items = array();
        foreach ($apis as $apiItem) {
            $newRow = array('s' => $apiItem['service']);

            $sl = strtolower($apiItem['service']);
            foreach ($devTypes as $devItem) {
                $mt = $devItem['level'];
                $newRow[$mt] = isset($rightsMap[$sl][$mt]) ? $rightsMap[$sl][$mt] : \PhalApi\DI()->config->get('app.project.default_app_api_rigths_is_allow'); // 默认无权限
            }

            $items[] = $newRow;
        }

        return array('items' => $items, 'dev_types' => $devTypes);
    }

    public function getRightsForDevApp($uid, $appKey) {
        // 全部接口
        $rsDomain = new RightsService();
        $apis = $rsDomain->listAllRightsService();

        $userDomain = new User();
        $level = $userDomain->getMemberLevel($uid);
        
        $appDomain = new Apps();
        $app = $appDomain->getApp($appKey, 'verify_status,app_status');

        // 规则配置
        $rights = RightsModel::model()->getList('member_type = ? OR uid = ? OR app_key = ?', array($level, $uid, $appKey), 'member_type,uid,app_key,service_lower,is_allow', NULL, 1, 10000);
        $rightsMap = array();
        foreach ($rights as $it) {
            $isAllow = $it['is_allow'] == 1 ? true : false;

            if (!isset($rightsMap[$it['service_lower']])) {
                $rightsMap[$it['service_lower']] = array(
                );
            }
            if (!empty($it['app_key'])) {
                $rightsMap[$it['service_lower']]['app_key'] = $isAllow;
            }
            if (!empty($it['uid'])) {
                $rightsMap[$it['service_lower']]['uid'] = $isAllow;
            }
            if (!empty($it['member_type'])) {
                $rightsMap[$it['service_lower']]['member_type'] = $isAllow;
            }
        }

        // 流量套餐
        $mpDomain = new Product();
        $products = $mpDomain->getBuyProductsSnapshot();
        $productsMap = array();
        foreach ($products as $it) {
            $productsMap[$it['product_key_lower']] = $it['id'];
        }

        $items = array();
        foreach ($apis as &$apiItemRef) {
            unset($apiItemRef['is_system']);

            $sl = strtolower($apiItemRef['service']);
            $apiItemRef['is_allow'] = false;
            $apiItemRef['iap'] = 0; // 命中的规则

            // 可购买的套餐ID
            $apiItemRef['product_id'] = isset($productsMap[$sl]) ? $productsMap[$sl] : 0;
            
            // 通过未通过 || 已注销
            if ($app['verify_status'] != 1 || $app['app_status'] == 0) {
                $apiItemRef['iap'] = 1; // 命中的规则
                continue;
            }
            
            // 优先级
            if (isset($rightsMap[$sl]['app_key'])) {
                $apiItemRef['is_allow'] = $rightsMap[$sl]['app_key'];
                $apiItemRef['iap'] = 2; // 命中的规则
                continue;
            }
            if (isset($rightsMap[$sl]['uid'])) {
                $apiItemRef['is_allow'] = $rightsMap[$sl]['uid'];
                $apiItemRef['iap'] = 3; // 命中的规则
                continue;
            }
            
            // 如果是管理员，默认有接口权限
            if ($level >= UserType::MEMBER_LEVEL_ADMIN) {
                $apiItemRef['is_allow'] = 1;
                $apiItemRef['iap'] = 4; // 命中的规则
                continue;
            }
            if (isset($rightsMap[$sl]['member_type'])) {
                $apiItemRef['is_allow'] = $rightsMap[$sl]['member_type'];
                $apiItemRef['iap'] = 5; // 命中的规则
                continue;
            }
        }

        return $apis;
    }

    public function assignRightsForDevTypeOrNot($rightsService, $memberType) {
        $rs = false;

        $where = array(
            'service_lower' => strtolower($rightsService),
            'member_type' => $memberType,
        );
        $rights = RightsModel::model()->getData($where, array(), 'id,is_allow');

        if ($rights) {
            // 原来有规则配置，翻转
            if ($rights['is_allow'] == 1) {
                $rights['is_allow'] = 0;
            } else {
                $rights['is_allow'] = 1;
                $rs = true;
            }
            // 更新规则
            RightsModel::model()->update($rights['id'], array('is_allow' => $rights['is_allow']));
        } else {
            $newData = array(
                'service' => $rightsService,
                'service_lower' => strtolower($rightsService),
                'member_type' => $memberType,
                'is_allow' => 1,
                'app_key' => '',
                'uid' => 0,
                'admin_uid' => \PhalApi\DI()->context->getUid(),
                'add_time' => date('Y-m-d H:i:s'),
            );
            $rs = true;
            RightsModel::model()->insert($newData);
        }

        return $rs;
    }

    public function searchRightsList($rightsService = '', $memberType = 0, $userId = 0, $appKey = '', $page = 1, $perpage = 100) {
        $where = array();
        if (!empty($rightsService)) {
            $where['service_lower LIKE ?'] = '%' . strtolower($rightsService) . '%';
        }
        if ($memberType > 0) {
            $where['member_type'] = $memberType;
        }
        if ($userId > 0) {
            $where['uid'] = $userId;
        }
        if (!empty($appKey)) {
            $where['app_key'] = $appKey;
        }
        $whereParams = array();
        $select = '*';
        $order = 'id DESC';

        $model = new RightsModel();
        $rs = $model-> getListEx($where, $whereParams, $select, $order, $page, $perpage);

        // 全部开发者类型
        $devTypes = UserType::getDeveloperLevelDropList(false);

        $rs['dev_types'] = $devTypes;

        return $rs;
    }

    public function assignRightsOrNot($id) {
        $rs = false;
        $model = new RightsModel();
        $rights = $model->get($id);
        if (!$rights) {
            return $rs;
        }
        if ($rights['is_allow'] == 1) {
            $rights['is_allow'] = 0;
        } else {
            $rights['is_allow'] = 1;
            $rs = true;
        }
        $model->update($id, array('is_allow' => $rights['is_allow']));

        return $rs;
    }

    public function delelteRightsRule($id) {
        $model = new RightsModel();
        return $model->delete($id);
    }

    public function createNewRightsRule($rightsService, $memberType = 0, $userId = 0, $appKey = '', $isAllow = true) {
        // 不能三个都不设置，授予对象不能全为空
        if ($memberType <= 0 && $userId <= 0 && empty($appKey)) {
            return false;
        }

        $where = array(
            'service_lower' => strtolower($rightsService),
            'member_type' => intval($memberType),
            'uid' => intval($userId),
            'app_key' => strval($appKey),
        );
        $rights = RightsModel::model()->getData($where);
        if ($rights) {
            return false;
        }

        $newData = $where;
        $newData['service'] = $rightsService;
        $newData['admin_uid'] = \PhalApi\DI()->context->getUid();
        $newData['add_time'] = date('Y-m-d H:i:s');
        $newData['is_allow'] = $isAllow ? 1 : 0;

        $id = RightsModel::model()->insert($newData);

        return $id > 0 ? true : false;
                            
    }
}
